use nom::bytes::complete::tag;
use nom::character::complete::anychar;
use nom::combinator::peek;
use nom::combinator::recognize;
use nom::multi::many_till;

use super::Context;
use crate::error::Res;
use crate::parser::exiting::ExitClass;
use crate::parser::parser_context::ContextElement;
use crate::parser::parser_context::ExitMatcherNode;
use crate::parser::parser_with_context::parser_with_context;
use crate::parser::plain_link::protocol;
use crate::parser::util::exit_matcher_parser;
use crate::parser::util::get_consumed;
use crate::parser::AngleLink;

#[tracing::instrument(ret, level = "debug")]
pub fn angle_link<'r, 's>(context: Context<'r, 's>, input: &'s str) -> Res<&'s str, AngleLink<'s>> {
    let (remaining, _) = tag("<")(input)?;
    let (remaining, proto) = protocol(context, remaining)?;
    let (remaining, _separator) = tag(":")(remaining)?;
    let (remaining, path) = path_angle(context, remaining)?;
    let (remaining, _) = tag(">")(remaining)?;
    let source = get_consumed(input, remaining);
    Ok((
        remaining,
        AngleLink {
            source,
            link_type: proto,
            path,
        },
    ))
}

#[tracing::instrument(ret, level = "debug")]
fn path_angle<'r, 's>(context: Context<'r, 's>, input: &'s str) -> Res<&'s str, &'s str> {
    let parser_context =
        context.with_additional_node(ContextElement::ExitMatcherNode(ExitMatcherNode {
            class: ExitClass::Beta,
            exit_matcher: &path_angle_end,
        }));

    let exit_matcher = parser_with_context!(exit_matcher_parser)(&parser_context);

    let (remaining, path) = recognize(many_till(anychar, peek(exit_matcher)))(input)?;
    Ok((remaining, path))
}

#[tracing::instrument(ret, level = "debug")]
fn path_angle_end<'r, 's>(context: Context<'r, 's>, input: &'s str) -> Res<&'s str, &'s str> {
    tag(">")(input)
}
