use nom::branch::alt;
use nom::bytes::complete::is_not;
use nom::bytes::complete::tag;
use nom::bytes::complete::tag_no_case;
use nom::character::complete::digit1;
use nom::character::complete::line_ending;
use nom::character::complete::space0;
use nom::character::complete::space1;
use nom::combinator::eof;
use nom::combinator::recognize;
use nom::combinator::verify;
use nom::sequence::tuple;

use super::Context;
use crate::error::Res;
use crate::parser::parser_with_context::parser_with_context;
use crate::parser::util::get_consumed;
use crate::parser::util::start_of_line;
use crate::parser::Clock;

#[tracing::instrument(ret, level = "debug")]
pub fn clock<'r, 's>(context: Context<'r, 's>, input: &'s str) -> Res<&'s str, Clock<'s>> {
    start_of_line(context, input)?;
    let (remaining, _leading_whitespace) = space0(input)?;
    let (remaining, _clock) = tag_no_case("clock:")(remaining)?;
    let (remaining, _gap_whitespace) = space1(remaining)?;

    let (remaining, _timestamp_junk) = alt((
        parser_with_context!(inactive_timestamp_range_duration)(context),
        parser_with_context!(inactive_timestamp)(context),
    ))(remaining)?;

    let source = get_consumed(input, remaining);
    Ok((remaining, Clock { source }))
}

#[tracing::instrument(ret, level = "debug")]
fn inactive_timestamp_range_duration<'r, 's>(
    context: Context<'r, 's>,
    input: &'s str,
) -> Res<&'s str, &'s str> {
    recognize(tuple((
        tag("["),
        is_not("\r\n]"),
        tag("]--["),
        is_not("\r\n]"),
        tag("]"),
        space1,
        tag("=>"),
        space1,
        digit1,
        tag(":"),
        verify(digit1, |mm: &str| mm.len() == 2),
        space0,
        alt((line_ending, eof)),
    )))(input)
}

#[tracing::instrument(ret, level = "debug")]
fn inactive_timestamp<'r, 's>(context: Context<'r, 's>, input: &'s str) -> Res<&'s str, &'s str> {
    recognize(tuple((
        tag("["),
        is_not("\r\n]"),
        tag("]"),
        space0,
        alt((line_ending, eof)),
    )))(input)
}
