use nom::branch::alt;
use nom::bytes::complete::tag;
use nom::character::complete::line_ending;
use nom::character::complete::space0;
use nom::combinator::eof;
use nom::combinator::recognize;
use nom::combinator::verify;
use nom::multi::many1_count;
use nom::sequence::tuple;

use super::Context;
use crate::error::Res;
use crate::parser::util::start_of_line;
use crate::parser::HorizontalRule;

#[tracing::instrument(ret, level = "debug")]
pub fn horizontal_rule<'r, 's>(
    context: Context<'r, 's>,
    input: &'s str,
) -> Res<&'s str, HorizontalRule<'s>> {
    start_of_line(context, input)?;
    let (remaining, rule) = recognize(tuple((
        space0,
        verify(many1_count(tag("-")), |dashes| *dashes >= 5),
        space0,
        alt((line_ending, eof)),
    )))(input)?;
    Ok((remaining, HorizontalRule { source: rule }))
}
